@@ -5,6 +5,7 @@ import pytest
 from sklearn.utils._testing import assert_almost_equal
 from sklearn.utils._testing import assert_array_almost_equal
 from sklearn.utils._testing import assert_allclose_dense_sparse
+from numpy.testing import assert_array_equal
 
 from sklearn import datasets
 from sklearn.decomposition import PCA, IncrementalPCA
@@ -25,21 +26,25 @@ def test_incremental_pca():
     X_transformed = ipca.fit_transform(X)
 
     assert X_transformed.shape == (X.shape[0], 2)
-    np.testing.assert_allclose(ipca.explained_variance_ratio_.sum(),
-                               pca.explained_variance_ratio_.sum(), rtol=1e-3)
+    np.testing.assert_allclose(
+        ipca.explained_variance_ratio_.sum(),
+        pca.explained_variance_ratio_.sum(),
+        rtol=1e-3,
+    )
 
     for n_components in [1, 2, X.shape[1]]:
         ipca = IncrementalPCA(n_components, batch_size=batch_size)
         ipca.fit(X)
         cov = ipca.get_covariance()
         precision = ipca.get_precision()
-        np.testing.assert_allclose(np.dot(cov, precision),
-                                   np.eye(X.shape[1]), atol=1e-13)
+        np.testing.assert_allclose(
+            np.dot(cov, precision), np.eye(X.shape[1]), atol=1e-13
+        )
 
 
 @pytest.mark.parametrize(
-    "matrix_class",
-    [sparse.csc_matrix, sparse.csr_matrix, sparse.lil_matrix])
+    "matrix_class", [sparse.csc_matrix, sparse.csr_matrix, sparse.lil_matrix]
+)
 def test_incremental_pca_sparse(matrix_class):
     # Incremental PCA on sparse arrays.
     X = iris.data
@@ -52,22 +57,29 @@ def test_incremental_pca_sparse(matrix_class):
     X_transformed = ipca.fit_transform(X_sparse)
 
     assert X_transformed.shape == (X_sparse.shape[0], 2)
-    np.testing.assert_allclose(ipca.explained_variance_ratio_.sum(),
-                               pca.explained_variance_ratio_.sum(), rtol=1e-3)
+    np.testing.assert_allclose(
+        ipca.explained_variance_ratio_.sum(),
+        pca.explained_variance_ratio_.sum(),
+        rtol=1e-3,
+    )
 
     for n_components in [1, 2, X.shape[1]]:
         ipca = IncrementalPCA(n_components, batch_size=batch_size)
         ipca.fit(X_sparse)
         cov = ipca.get_covariance()
         precision = ipca.get_precision()
-        np.testing.assert_allclose(np.dot(cov, precision),
-                                   np.eye(X_sparse.shape[1]), atol=1e-13)
+        np.testing.assert_allclose(
+            np.dot(cov, precision), np.eye(X_sparse.shape[1]), atol=1e-13
+        )
 
     with pytest.raises(
-            TypeError,
-            match="IncrementalPCA.partial_fit does not support "
+        TypeError,
+        match=(
+            "IncrementalPCA.partial_fit does not support "
             "sparse input. Either convert data to dense "
-            "or use IncrementalPCA.fit to do so in batches."):
+            "or use IncrementalPCA.fit to do so in batches."
+        ),
+    ):
         ipca.partial_fit(X_sparse)
 
 
@@ -75,7 +87,7 @@ def test_incremental_pca_check_projection():
     # Test that the projection of data is correct.
     rng = np.random.RandomState(1999)
     n, p = 100, 3
-    X = rng.randn(n, p) * .1
+    X = rng.randn(n, p) * 0.1
     X[:10] += np.array([3, 4, 5])
     Xt = 0.1 * rng.randn(1, p) + np.array([3, 4, 5])
 
@@ -85,11 +97,11 @@ def test_incremental_pca_check_projection():
     Yt = IncrementalPCA(n_components=2).fit(X).transform(Xt)
 
     # Normalize
-    Yt /= np.sqrt((Yt ** 2).sum())
+    Yt /= np.sqrt((Yt**2).sum())
 
     # Make sure that the first element of Yt is ~1, this means
     # the reconstruction worked as expected
-    assert_almost_equal(np.abs(Yt[0][0]), 1., 1)
+    assert_almost_equal(np.abs(Yt[0][0]), 1.0, 1)
 
 
 def test_incremental_pca_inverse():
@@ -97,7 +109,7 @@ def test_incremental_pca_inverse():
     rng = np.random.RandomState(1999)
     n, p = 50, 3
     X = rng.randn(n, p)  # spherical data
-    X[:, 1] *= .00001  # make middle component relatively small
+    X[:, 1] *= 0.00001  # make middle component relatively small
     X += [5, 4, 3]  # make a large mean
 
     # same check that we can find the original data from the transformed
@@ -112,19 +124,28 @@ def test_incremental_pca_validation():
     # Test that n_components is >=1 and <= n_features.
     X = np.array([[0, 1, 0], [1, 0, 0]])
     n_samples, n_features = X.shape
-    for n_components in [-1, 0, .99, 4]:
-        with pytest.raises(ValueError, match="n_components={} invalid"
-                           " for n_features={}, need more rows than"
-                           " columns for IncrementalPCA"
-                           " processing".format(n_components,
-                                                n_features)):
+    for n_components in [-1, 0, 0.99, 4]:
+        with pytest.raises(
+            ValueError,
+            match=(
+                "n_components={} invalid"
+                " for n_features={}, need more rows than"
+                " columns for IncrementalPCA"
+                " processing".format(n_components, n_features)
+            ),
+        ):
             IncrementalPCA(n_components, batch_size=10).fit(X)
 
     # Tests that n_components is also <= n_samples.
     n_components = 3
-    with pytest.raises(ValueError, match="n_components={} must be"
-                       " less or equal to the batch number of"
-                       " samples {}".format(n_components, n_samples)):
+    with pytest.raises(
+        ValueError,
+        match=(
+            "n_components={} must be"
+            " less or equal to the batch number of"
+            " samples {}".format(n_components, n_samples)
+        ),
+    ):
         IncrementalPCA(n_components=n_components).partial_fit(X)
 
 
@@ -225,8 +246,7 @@ def test_incremental_pca_batch_rank():
         ipca = IncrementalPCA(n_components=20, batch_size=batch_size).fit(X)
         all_components.append(ipca.components_)
 
-    for components_i, components_j in zip(all_components[:-1],
-                                          all_components[1:]):
+    for components_i, components_j in zip(all_components[:-1], all_components[1:]):
         assert_allclose_dense_sparse(components_i, components_j)
 
 
@@ -235,7 +255,7 @@ def test_incremental_pca_partial_fit():
     rng = np.random.RandomState(1999)
     n, p = 50, 3
     X = rng.randn(n, p)  # spherical data
-    X[:, 1] *= .00001  # make middle component relatively small
+    X[:, 1] *= 0.00001  # make middle component relatively small
     X += [5, 4, 3]  # make a large mean
 
     # same check that we can find the original data from the transformed
@@ -275,19 +295,21 @@ def test_incremental_pca_against_pca_random_data():
 
 def test_explained_variances():
     # Test that PCA and IncrementalPCA calculations match
-    X = datasets.make_low_rank_matrix(1000, 100, tail_strength=0.,
-                                      effective_rank=10, random_state=1999)
+    X = datasets.make_low_rank_matrix(
+        1000, 100, tail_strength=0.0, effective_rank=10, random_state=1999
+    )
     prec = 3
     n_samples, n_features = X.shape
     for nc in [None, 99]:
         pca = PCA(n_components=nc).fit(X)
         ipca = IncrementalPCA(n_components=nc, batch_size=100).fit(X)
-        assert_almost_equal(pca.explained_variance_, ipca.explained_variance_,
-                            decimal=prec)
-        assert_almost_equal(pca.explained_variance_ratio_,
-                            ipca.explained_variance_ratio_, decimal=prec)
-        assert_almost_equal(pca.noise_variance_, ipca.noise_variance_,
-                            decimal=prec)
+        assert_almost_equal(
+            pca.explained_variance_, ipca.explained_variance_, decimal=prec
+        )
+        assert_almost_equal(
+            pca.explained_variance_ratio_, ipca.explained_variance_ratio_, decimal=prec
+        )
+        assert_almost_equal(pca.noise_variance_, ipca.noise_variance_, decimal=prec)
 
 
 def test_singular_values():
@@ -297,36 +319,42 @@ def test_singular_values():
     n_samples = 1000
     n_features = 100
 
-    X = datasets.make_low_rank_matrix(n_samples, n_features, tail_strength=0.0,
-                                      effective_rank=10, random_state=rng)
+    X = datasets.make_low_rank_matrix(
+        n_samples, n_features, tail_strength=0.0, effective_rank=10, random_state=rng
+    )
 
-    pca = PCA(n_components=10, svd_solver='full', random_state=rng).fit(X)
+    pca = PCA(n_components=10, svd_solver="full", random_state=rng).fit(X)
     ipca = IncrementalPCA(n_components=10, batch_size=100).fit(X)
     assert_array_almost_equal(pca.singular_values_, ipca.singular_values_, 2)
 
     # Compare to the Frobenius norm
     X_pca = pca.transform(X)
     X_ipca = ipca.transform(X)
-    assert_array_almost_equal(np.sum(pca.singular_values_**2.0),
-                              np.linalg.norm(X_pca, "fro")**2.0, 12)
-    assert_array_almost_equal(np.sum(ipca.singular_values_**2.0),
-                              np.linalg.norm(X_ipca, "fro")**2.0, 2)
+    assert_array_almost_equal(
+        np.sum(pca.singular_values_**2.0), np.linalg.norm(X_pca, "fro") ** 2.0, 12
+    )
+    assert_array_almost_equal(
+        np.sum(ipca.singular_values_**2.0), np.linalg.norm(X_ipca, "fro") ** 2.0, 2
+    )
 
     # Compare to the 2-norms of the score vectors
-    assert_array_almost_equal(pca.singular_values_,
-                              np.sqrt(np.sum(X_pca**2.0, axis=0)), 12)
-    assert_array_almost_equal(ipca.singular_values_,
-                              np.sqrt(np.sum(X_ipca**2.0, axis=0)), 2)
+    assert_array_almost_equal(
+        pca.singular_values_, np.sqrt(np.sum(X_pca**2.0, axis=0)), 12
+    )
+    assert_array_almost_equal(
+        ipca.singular_values_, np.sqrt(np.sum(X_ipca**2.0, axis=0)), 2
+    )
 
     # Set the singular values and see what we get back
     rng = np.random.RandomState(0)
     n_samples = 100
     n_features = 110
 
-    X = datasets.make_low_rank_matrix(n_samples, n_features, tail_strength=0.0,
-                                      effective_rank=3, random_state=rng)
+    X = datasets.make_low_rank_matrix(
+        n_samples, n_features, tail_strength=0.0, effective_rank=3, random_state=rng
+    )
 
-    pca = PCA(n_components=3, svd_solver='full', random_state=rng)
+    pca = PCA(n_components=3, svd_solver="full", random_state=rng)
     ipca = IncrementalPCA(n_components=3, batch_size=100)
 
     X_pca = pca.fit_transform(X)
@@ -343,14 +371,14 @@ def test_singular_values():
 
 def test_whitening():
     # Test that PCA and IncrementalPCA transforms match to sign flip.
-    X = datasets.make_low_rank_matrix(1000, 10, tail_strength=0.,
-                                      effective_rank=2, random_state=1999)
+    X = datasets.make_low_rank_matrix(
+        1000, 10, tail_strength=0.0, effective_rank=2, random_state=1999
+    )
     prec = 3
     n_samples, n_features = X.shape
     for nc in [None, 9]:
         pca = PCA(whiten=True, n_components=nc).fit(X)
-        ipca = IncrementalPCA(whiten=True, n_components=nc,
-                              batch_size=250).fit(X)
+        ipca = IncrementalPCA(whiten=True, n_components=nc, batch_size=250).fit(X)
 
         Xt_pca = pca.transform(X)
         Xt_ipca = ipca.transform(X)
@@ -382,5 +410,29 @@ def test_incremental_pca_partial_fit_float_division():
     pca2.partial_fit(B)
     singular_vals_int_samples_seen = pca2.singular_values_
 
-    np.testing.assert_allclose(singular_vals_float_samples_seen,
-                               singular_vals_int_samples_seen)
+    np.testing.assert_allclose(
+        singular_vals_float_samples_seen, singular_vals_int_samples_seen
+    )
+
+
+def test_incremental_pca_fit_overflow_error():
+    # Test for overflow error on Windows OS
+    # (non-regression test for issue #17693)
+    rng = np.random.RandomState(0)
+    A = rng.rand(500000, 2)
+
+    ipca = IncrementalPCA(n_components=2, batch_size=10000)
+    ipca.fit(A)
+
+    pca = PCA(n_components=2)
+    pca.fit(A)
+
+    np.testing.assert_allclose(ipca.singular_values_, pca.singular_values_)
+
+
+def test_incremental_pca_feature_names_out():
+    """Check feature names out for IncrementalPCA."""
+    ipca = IncrementalPCA(n_components=2).fit(iris.data)
+
+    names = ipca.get_feature_names_out()
+    assert_array_equal([f"incrementalpca{i}" for i in range(2)], names)
